package ru.dbotthepony.mc.otm.core

import java.util.LinkedList

class TimerQueue {
	private var ticks = 0
	private val list = LinkedList<Timer>()

	inner class Timer(val timerTicks: Int, val runnable: Runnable) {
		val ringAt = ticks + timerTicks

		var finished = false
			private set

		init {
			if (list.isEmpty()) {
				list.addLast(this)
			} else {
				val iterator = list.listIterator()
				var hit = false

				for (value in iterator) {
					if (value.ringAt == ringAt) {
						hit = true
						iterator.add(this)
						break
					} else if (value.ringAt > ringAt) {
						if (iterator.hasPrevious()) {
							iterator.previous()
							iterator.add(this)
						} else {
							list.addFirst(this)
						}

						hit = true
						break
					}
				}

				if (!hit) {
					list.addLast(this)
				}
			}
		}

		fun execute() {
			check(!finished) { "Already finished" }
			runnable.run()
			finished = true
		}
	}

	fun tick() {
		ticks++

		while (list.isNotEmpty()) {
			val head = list.first

			if (head.ringAt <= ticks) {
				head.execute()
				list.removeFirst()
			} else {
				break
			}
		}
	}

	fun clear() {
		ticks = 0
		list.clear()
	}
}
